import './math.dart';
import 'key_space.dart';

List<String> _generateExtraKeys(int keyCount) {
  keyCount %= keys.length;

  final result = <String>[];
  for (int i = 1; i <= keyCount; i++) {
    final index = (keys.length / (keyCount + 1)) * i;
    result.add(keys[index.roundCorrectly()]);
  }
  return result;
}

/// Permutation with repetition
Iterable<String> _generateFoundation(int n, [String str = '']) sync* {
  if (n > 0) {
    for (int i = 0; i < keys.length; i++) {
      yield* _generateFoundation(n - 1, str + keys[i]);
    }
  } else {
    yield str;
  }
}

String _removeTrailings(String value, String trailingCharacter) {
  var last = value.length - 1;
  while (last >= 0 && value[last] == trailingCharacter) {
    last--;
  }
  return value.substring(0, last + 1);
}

Iterable<String> _generate(int count) sync* {
  final chars = logBase(count, keys.length).floor();
  final avgAdditionalInterpoints =
      ((count + 1) / (pow(keys.length, chars))) - 1;
  final weight = avgAdditionalInterpoints % 1;
  final a = avgAdditionalInterpoints.toInt();
  var stack = 0.5;

  final additional = [
    _generateExtraKeys(a),
    _generateExtraKeys(a + 1),
  ];

  final base = _generateFoundation(chars).toList();

  for (int i = 0; i < base.length; i++) {
    final String left = base[i];
    yield _removeTrailings(left, keys.first);
    yield* additional[stack.floor()].map((x) => left + x);
    stack %= 1;
    stack += weight;
  }
}

/// Generate strings that satisfy the follwing constraints:
/// 1. the lexicographical order.
/// 2. minimal use of characters.
/// 3. equally-spaced to use as few additional characters as possible in the future.
///
/// If you have an ordered database table or collection for which you want to switch
/// to an efficient ordering system, generate order keys with this.
Iterable<String> generateOrderKeys(int count) sync* {
  if (count <= 0) {
    return;
  } else if (count < keys.length) {
    yield* _generateExtraKeys(count);
  } else {
    yield* _generate(count).skip(1);
  }
}
